@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(IO::Async::Notifier);
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ IO::AsyncX::System - fork+exec, capturing STDOUT/STDERR
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -77,12 +77,15 @@ sub run {
 	my $stdout = [];
 	my $stderr = [];
 	my $stdin = [];
-	my $stdio_def = {
+	my $stdin_def = {
 		(
-			defined($stdin)
-			? (from => join "\n", @$stdin)
-			: ()
+			# Allow both ['x','y'] and "x\ny" as input, although we only document the former
+			defined($args{stdin})
+			? (from => ref($args{stdin}) ? join "\n", @{delete $args{stdin}} : $args{stdin})
+			: (from => '')
 		),
+	};
+	my $stdout_def = {
 		on_read => (
 			$args{utf8}
 			? $self->curry::read_utf8($stdout)
@@ -99,7 +102,8 @@ sub run {
 	my $f = $self->loop->new_future;
 	my $proc = IO::Async::Process->new(
 		command => $cmd,
-		stdio => $stdio_def,
+		stdin => $stdin_def,
+		stdout => $stdout_def,
 		stderr => $stderr_def,
 		on_finish => sub { $f->done($_[1], $stdout, $stderr) unless $f->is_ready },
 		on_exception => sub { $f->fail($_[1]) unless $f->is_ready },
@@ -112,7 +116,7 @@ sub run {
 		$self->loop->timeout_future(after => $args{timeout})->on_fail(
 			sub { $proc->kill(9) if $proc->is_running }
 		)
-	);
+	)->on_cancel(sub { $f->cancel unless $f->is_ready });
 }
 
 sub read_binary {
@@ -130,6 +134,7 @@ sub read_utf8 {
 	push @$target, decode_utf8($$buf) if $eof;
 	0
 }
+
 1;
 
 __END__
